import axios from 'axios'
import { uploadUrl } from '@/utils/request'

// 活动类型
export const NOTICE_ACTIVITY_TYPE = 'NOTICE' // 公告的活动类型
export const BUSINESS_ACTIVITY_TYPE = 'BUSINESS' // 商机的活动类型

// 不需要结束时间/等乱七八糟的
export const NOT_LUAN_QI_BA_ZAO = [NOTICE_ACTIVITY_TYPE, BUSINESS_ACTIVITY_TYPE]

const DEFAULT_PEOPLE_NUMBER = 10000
const DEFAULT_END_TIME = '2099-01-31 00:00:00'
export const DEFAULT_ACTIVITY_ADDRESS = 'xxxxxxxxxxxxxxxxx'

// 活动类型
export const ACTIVITY_TYPE_INFO = [
  { name: '公益活动', enumKey: 'PUBLIC', typeKey: 1 },
  { name: '会议', enumKey: 'MEETING', typeKey: 2 },
  { name: '爱心传递', enumKey: 'LOVE', typeKey: 3 },
  { name: '公告', enumKey: NOTICE_ACTIVITY_TYPE, typeKey: 4 },
  { name: '商机', enumKey: BUSINESS_ACTIVITY_TYPE, typeKey: 5 }
]

// 活动状态
// !内容顺序不要更改
export const ACTIVITY_STATUS_INFO = [
  { name: '草稿', enumKey: 'DRAFT', typeKey: 1 },
  { name: '已开始', enumKey: 'STARTED', typeKey: 2 },
  { name: '已结束', enumKey: 'END', typeKey: 3 },
  { name: '已撤销', enumKey: 'CANCELLED', typeKey: 4 }
]

/**
 * @description 获取创建活动的默认表单数据
 * @returns
 */

export const getDefaultActivityForm = (isMerge = false, originData = {}) => {
  const defaultActivity = {
    id: undefined,
    typeEnum: NOTICE_ACTIVITY_TYPE,
    title: '',
    cover: '',
    content: '',
    peopleNumber: DEFAULT_PEOPLE_NUMBER,
    stateEnum: '',
    registrationScope: 1,
    promotionalVideo: '',
    startTime: '',
    endTime: DEFAULT_END_TIME,
    activitiesAddress: DEFAULT_ACTIVITY_ADDRESS,
    participationMethod: '',
    activitiesDetailAddress: '',
    precautions: '',
    opportunityType: '',
    opportunityContactNumber: '',
    opportunityContactEmail: '',
    opportunityAuthorityUrl: '',
    publishMemberName: '',
    publishMemberAvatar: ''
  }

  if (isMerge) {
    const { club, activities } = originData
    return {
      club: {
        id: club.id || undefined
      },
      activities: {
        ...defaultActivity,
        ...activities,
        typeEnum: activities.typeEnum || defaultActivity.typeEnum,
        peopleNumber: NOT_LUAN_QI_BA_ZAO.includes(activities.typeEnum) ? DEFAULT_PEOPLE_NUMBER : activities.peopleNumber !== DEFAULT_PEOPLE_NUMBER ? activities.peopleNumber : '',
        endTime: NOT_LUAN_QI_BA_ZAO.includes(activities.typeEnum) ? DEFAULT_END_TIME : activities.endTime !== DEFAULT_END_TIME ? activities.endTime : '',
        activitiesAddress: NOT_LUAN_QI_BA_ZAO.includes(activities.typeEnum) ? DEFAULT_ACTIVITY_ADDRESS : activities.activitiesAddress !== DEFAULT_ACTIVITY_ADDRESS ? activities.activitiesAddress : ''
      }
    }
  }

  return {
    club: { id: undefined },
    activities: defaultActivity
  }
}

/**
 * @description 获取当前表单校验rules
 */
export const getCurrentActivityRules = (typeEnum) => {
  const baseOneRules = {
    'club.id': [{ required: true, message: '请选择俱乐部', trigger: 'change' }],
    'activities.typeEnum': [{ required: true, message: '请选择类型', trigger: 'change' }],
    'activities.title': [{ required: true, message: '请填写标题', trigger: 'blur' }],
    'activities.startTime': [{ required: true, message: '请选择开始时间', trigger: 'blur' }],
    'activities.endTime': [{ required: true, message: '请选择结束时间', trigger: 'blur' }],
    'activities.registrationScope': [{ required: true, message: '请选择范围', trigger: 'blur' }],
    'activities.peopleNumber': [{ required: true, message: '请填写参与人数', trigger: 'blur' }]
  }

  const baseTwoRules = {
    'activities.content': [{ required: true, message: '请输入内容', trigger: 'blur' }]
  }

  return { oneRules: baseOneRules, twoRules: baseTwoRules }
}

/**
 * @description 拦截quill的图片上传操作
 * @param {VueInstance} ctx 当前页面实例
 * @param {Object} options 自定义配置
 * @param {boolean} options.showLoading 是否要显示图片上传的加载蒙版
 *
 */
export const interceptQuillImageUpload = (ctx, options = {}) => {
  const quill = ctx.$refs.quillEditorRef.quill
  if (!quill) return

  const _customImageUpload = async (file) => {
    const formData = new FormData()
    formData.append('file', file)
    const res = await axios.post(uploadUrl, formData)
    if (!res.data.code) {
      return res.data.data.url
    } else {
      ctx.$message.error('图片上传失败')
      return undefined
    }
  }

  const _onImageUpload = () => {
    const input = document.createElement('input')
    input.setAttribute('type', 'file')
    input.setAttribute('accept', 'image/*')

    input.onchange = async () => {
      const file = input.files[0]

      if (file) {
        try {
          options.showLoading = true
          const allowedExtensions = /(\.png|\.jpg|\.jpeg)$/i
          if (!allowedExtensions.exec(file.name)) {
            return ctx.$message.error('上传的文件格式不正确，请选择 .png, .jpg 或 .jpeg 格式的文件。')
          }
          // 检查文件大小
          const maxSize = 5 * 1024 * 1024
          if (file.size > maxSize) {
            return ctx.$message.error('文件大小不能超过 5MB。')
          }
          const imageUrl = await _customImageUpload(file)
          const range = quill.getSelection(true)
          quill.insertEmbed(range.index, 'image', imageUrl)
          quill.setSelection(range.index + 1)
          ctx.$message.success('图片上传成功')
        } catch (error) {
          ctx.$message.error('图片上传失败')
        } finally {
          options.showLoading = false
        }
      }
    }

    input.click()
  }

  const toolbar = quill.getModule('toolbar')
  toolbar.addHandler('image', _onImageUpload)
}

/**
 * @description 监听 输入框的高度变化，然后动态修改走马灯的高度
 */
export const listenEditorContentHeightChange = (ctx, calcHeightFn) => {
  if (!ctx || !calcHeightFn) {
    return console.error('this and calcHeightFn is required')
  }
  const quill = ctx.$refs.quillEditorRef.quill
  if (!quill) return
  const editorElement = ctx.$refs.quillEditorRef.$el.querySelector('.ql-editor')
  let prevHeight = 0
  const observer = new MutationObserver(() => {
    const currentHeight = editorElement.clientHeight
    if (currentHeight > prevHeight) {
      prevHeight = currentHeight
      typeof calcHeightFn === 'function' && calcHeightFn(currentHeight)
    }
  })
  observer.observe(editorElement, { childList: true, subtree: true, characterData: true })
  return () => {
    observer.disconnect(editorElement)
  }
}

export const getTypeEnumByValue = (value) => {
  if (!value) return ''
  return (ACTIVITY_TYPE_INFO.find((item) => item.typeKey === value) || {}).enumKey || ''
}

export const getStateEnumByValue = (value) => {
  if (!value) return ''
  return (ACTIVITY_STATUS_INFO.find((item) => item.typeKey === value) || {}).enumKey || ''
}
